import { debounce } from "lodash";
import {
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  Grid,
  Typography,
  Box,
  FormControl,
  InputLabel,
  Select,
  MenuItem,
  TextField,
  TableContainer,
  Paper,
  Table,
  TableHead,
  TableRow,
  TableCell,
  TableBody,
  Button,
  TablePagination,
  Skeleton,
} from "@mui/material";
import axios from "axios";
import React, { useCallback, useEffect, useState, useRef } from "react";
import { Link, useParams, useNavigate } from "react-router-dom";
import swal from "sweetalert";
import { isAutheticated } from "src/auth";
import InvoiceTable from "../orders/invoiceTable";
import PendingOrderTable from "../orders/pendingOrderTable";

const SinglePrincipalDistributorAllDetails = () => {
  const [user, setUser] = useState(null);
  const [userOrder, setUserOrder] = useState({ totalOrders: 0, totalValue: 0 });
  const [userAllAddress, setUserAllAddress] = useState([]);
  const [orders, setOrders] = useState([]);
  const [totalOrders, setTotalOrders] = useState(0);
  const [page, setPage] = useState(0);
  const [rowsPerPage, setRowsPerPage] = useState(10);
  const [loading, setLoading] = useState(true);
  const [searchField, setSearchField] = useState("Order ID");
  const [searchText, setSearchText] = useState("");

  const token = isAutheticated();
  const { _id } = useParams();
  const searchOrderIdRef = useRef("");
  const searchStatusRef = useRef("");

  // Debounced function to fetch orders
  const fetchOrdersDebounced = useRef(
    debounce((page, limit, orderId, status) => {
      fetchOrders(page, limit, orderId, status);
    }, 500)
  ).current;

  const handleSearchFieldChange = (event) => {
    setSearchField(event.target.value);
    setSearchText("");
  };

  const handleSearchChange = (event) => {
    setSearchText(event.target.value);
    if (searchField === "Order ID") {
      searchOrderIdRef.current = event.target.value;
    } else {
      searchStatusRef.current = event.target.value;
    }
    // Call the debounced function to fetch orders
    fetchOrdersDebounced(
      page + 1,
      rowsPerPage,
      searchOrderIdRef.current,
      searchStatusRef.current
    );
  };
  const [openTMModal, setOpenTMModal] = useState(false);
  const [singleorder, setSingleOrder] = useState(null); // State to hold fetched order details

  const handleCloseTMModal = () => {
    setOpenTMModal(false);
    setSingleOrder(null); // Clear the order details when closing the modal
  };

  // Function to fetch order details
  const fetchOrderDetails = async (id) => {
    try {
      const response = await axios.get(`/api/get-single-placed-order-pd/${id}`, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
      });
      setSingleOrder(response.data?.singleOrder);
      setOpenTMModal(true); 
    } catch (error) {
      console.error('Error fetching order details:', error);
     
    }
  };
  
  // Fetch Shipping address of the individual user
  const getUserAddress = useCallback(async () => {
    try {
      const response = await axios.get(
        `/api/shipping/address/user/address/${_id}`,
        {
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );
      setUserAllAddress(response.data?.UserShippingAddress || []);
    } catch (error) {
      swal({
        title: "Warning",
        text: error.message,
        icon: "error",
        button: "Close",
        dangerMode: true,
      });
    }
  }, [_id, token]);

  // Fetch Order Count and Total Value
  const getOrdersCount = useCallback(async () => {
    try {
      const response = await axios.get(`/api/single-pd-ordercount/${_id}`, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
      });
      setUserOrder(response.data);
    } catch (error) {
      swal({
        title: "Warning",
        text: error.message,
        icon: "error",
        button: "Close",
        dangerMode: true,
      });
    }
  }, [_id, token]);

  // Fetch User Details
  const getUserDetails = useCallback(async () => {
    try {
      const response = await axios.get(`/api/v1/admin/user/${_id}`, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
      });
      setUser(response.data.user);
    } catch (error) {
      swal({
        title: "Warning",
        text: error.message,
        icon: "error",
        button: "Close",
        dangerMode: true,
      });
    }
  }, [_id, token]);

  // Fetch Orders with Pagination, Order ID, and Status search
  const fetchOrders = useCallback(async () => {
    setLoading(true);
    try {
      const response = await axios.get(`/api/single-pd-order/${_id}`, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
        params: {
          page: page + 1,
          limit: rowsPerPage,
          orderId: searchOrderIdRef.current,
          status: searchStatusRef.current,
        },
      });
      setOrders(response.data.orders || []);
      setTotalOrders(response.data.totalOrders || 0);
    } catch (error) {
      swal({
        title: "Warning",
        text: error.message,
        icon: "error",
        button: "Close",
        dangerMode: true,
      });
    } finally {
      setLoading(false);
    }
  }, [_id, token, page, rowsPerPage]);

  useEffect(() => {
    getOrdersCount();
    getUserAddress();
    getUserDetails();
    fetchOrders();
  }, [_id, getOrdersCount, getUserAddress, getUserDetails, fetchOrders]);

  const handleChangePage = (event, newPage) => {
    setPage(newPage);
    // Fetch orders whenever page changes
    fetchOrdersDebounced(
      newPage + 1,
      rowsPerPage,
      searchOrderIdRef.current,
      searchStatusRef.current
    );
  };

  const handleChangeRowsPerPage = (event) => {
    setRowsPerPage(parseInt(event.target.value));
    setPage(0);
    // Fetch orders with the new rows per page setting
    fetchOrdersDebounced(
      1,
      parseInt(event.target.value),
      searchOrderIdRef.current,
      searchStatusRef.current
    );
  };
  return (
    <div>
      <div className="row">
        <div className="col-12">
          <div className="page-title-box d-flex align-items-center justify-content-between">
            <div style={{ fontSize: "22px" }} className="fw-bold">
              Principal Distributor All Details
            </div>
            <div className="page-title-right">
              <Link to="/principal-distributor">
                <Button
                  variant="contained"
                  color="secondary"
                  style={{
                    fontWeight: "bold",
                    marginBottom: "1rem",
                    textTransform: "capitalize",
                  }}
                >
                  Back
                </Button>
              </Link>
            </div>
          </div>
        </div>
      </div>
      <div className="card" style={{ padding: "1rem" }}>
        <h5 style={{ fontWeight: "bold" }}>
          &bull; Principal Distributor Profile
        </h5>
        <div style={{ marginLeft: "1rem", marginTop: "1rem" }}>
          <Typography style={{ fontWeight: "bold", fontSize: "1.2rem" }}>
            Principal Distributor ID:
            <Typography
              component="span"
              style={{ fontWeight: "normal", marginLeft: "0.5rem" }}
            >
              {user?.uniqueId}
            </Typography>
          </Typography>
          {/* Repeating fields with similar styling and structure */}
          <div style={{ display: "flex", justifyContent: "space-between" }}>
            <div style={{ flex: 1, paddingRight: "1rem" }}>
              {[
                { label: "SBU", value: user?.SBU },
                { label: "Name", value: user?.name },
                { label: "Email", value: user?.email },
                { label: "Mobile Number", value: user?.phone },
              ].map((item, index) => (
                <Typography
                  key={index}
                  style={{ fontWeight: "bold", fontSize: "1.2rem" }}
                >
                  {item.label}:
                  <Typography
                    component="span"
                    style={{ fontWeight: "normal", marginLeft: "0.5rem" }}
                  >
                    {item.value}
                  </Typography>
                </Typography>
              ))}
            </div>
            <div style={{ flex: 1, paddingLeft: "1rem" }}>
              {[
                {
                  label: "Date Registered",
                  value: new Date(user?.createdAt).toLocaleString("en-IN", {
                    weekday: "short",
                    month: "short",
                    day: "numeric",
                    year: "numeric",
                    hour: "numeric",
                    minute: "numeric",
                    hour12: true,
                  }),
                },
                {
                  label: "Last Purchase",
                  value:
                    orders.length > 0
                      ? new Date(orders[0]?.createdAt).toLocaleString("en-IN", {
                          weekday: "short",
                          month: "short",
                          day: "numeric",
                          year: "numeric",
                          hour: "numeric",
                          minute: "numeric",
                          hour12: true,
                        })
                      : "No Purchase",
                },
                { label: "Total Orders", value: userOrder?.totalOrders },
                { label: "Total Spent", value: `₹ ${userOrder?.totalValue}` },
              ].map((item, index) => (
                <Typography
                  key={index}
                  style={{ fontWeight: "bold", fontSize: "1.2rem" }}
                >
                  {item.label}:
                  <Typography
                    component="span"
                    style={{ fontWeight: "normal", marginLeft: "0.5rem" }}
                  >
                    {item.value}
                  </Typography>
                </Typography>
              ))}
            </div>
          </div>
        </div>
        <div style={{ marginTop: "2rem" }}>
          <h5 style={{ fontWeight: "bold", marginBottom: "1rem" }}>
            &bull; Addresses{" "}
          </h5>{" "}
          <h5 style={{ fontWeight: "bold", marginLeft: "1rem" }}>
            &bull; Total Addresses : {userAllAddress?.length}{" "}
          </h5>
          {userAllAddress?.length > 0 && (
            <div className="table-responsive table-shoot mt-3">
              <table
                className="table table-centered table-nowrap"
                style={{ border: "1px solid" }}
              >
                <thead
                  className="thead-info"
                  style={{ background: "rgb(140, 213, 213)" }}
                >
                  <tr>
                    <th>SL No.</th>
                    <th>Address </th>
                    {/* <th>Profile Image</th> */}
                  </tr>
                </thead>
                <tbody>
                  {userAllAddress?.length === 0 && (
                    <tr className="text-center">
                      <td colSpan="6">
                        <h5>No Data Available</h5>
                      </td>
                    </tr>
                  )}
                  {userAllAddress?.map((address, i) => {
                    return (
                      <tr key={i}>
                        <td className="text-start">{i + 1}</td>
                        <td style={{ maxWidth: "400px" }}>
                          <strong>
                            {address?.first_Name} {address?.last_name}
                            {address.tradeName
                              ? `${address.tradeName},`
                              : "No tradeName "}
                            {address.gstNumber ? `${address.gstNumber},` : ""}
                            {address?.street},{address?.city},{address?.state},
                            {address?.country},{address?.postalCode}
                          </strong>
                        </td>
                      </tr>
                    );
                  })}
                </tbody>
              </table>
            </div>
          )}
        </div>
        <div style={{ marginTop: "2rem" }}>
          <h5 style={{ fontWeight: "bold" }}>&bull; Orders</h5>
          <Box className="mt-3">
            <Box display="flex" mb={2} alignItems="center">
              <FormControl variant="outlined" sx={{ minWidth: 150, mr: 2 }}>
                <InputLabel id="search-field-label">Search By</InputLabel>
                <Select
                  labelId="search-field-label"
                  id="search-field"
                  value={searchField}
                  onChange={handleSearchFieldChange}
                  label="Search By"
                >
                  <MenuItem value="Order ID">Order ID</MenuItem>
                  <MenuItem value="Status">Status</MenuItem>
                </Select>
              </FormControl>
              <TextField
                label={`Search by ${searchField}`}
                variant="outlined"
                value={searchText}
                onChange={handleSearchChange}
                fullWidth
              />
            </Box>
            <TableContainer component={Paper}>
              <Table>
                <TableHead>
                  <TableRow>
                    <TableCell>Order ID</TableCell>
                    <TableCell>Order Date</TableCell>
                    <TableCell>Items</TableCell>
                    <TableCell>Order Value</TableCell>
                    <TableCell>Status</TableCell>
                    <TableCell>Action</TableCell>
                  </TableRow>
                </TableHead>
                <TableBody>
                  {loading ? (
                    <TableRow>
                      <TableCell colSpan={6} align="center">
                        Loading...
                      </TableCell>
                    </TableRow>
                  ) : orders.length === 0 ? (
                    <TableRow>
                      <TableCell colSpan={6} align="center">
                        No Orders Found
                      </TableCell>
                    </TableRow>
                  ) : (
                    orders.map((order) => (
                      <TableRow key={order._id}>
                        <TableCell>{order.uniqueId}</TableCell>
                        <TableCell>
                          {new Date(order.createdAt).toLocaleString()}
                        </TableCell>
                        <TableCell>{order.orderItem.length}</TableCell>
                        <TableCell>₹ {order.grandTotal}</TableCell>
                        <TableCell>{order.status}</TableCell>
                        <TableCell>
                          <Button
                            variant="contained"
                            color="primary"
                            onClick={() => fetchOrderDetails(order._id)}
                          >
                            View
                          </Button>
                        </TableCell>
                      </TableRow>
                    ))
                  )}
                </TableBody>
              </Table>
            </TableContainer>
            {/* Pagination */}
            <TablePagination
              rowsPerPageOptions={[10, 25, 50]}
              component="div"
              count={totalOrders}
              rowsPerPage={rowsPerPage}
              page={page}
              onPageChange={handleChangePage}
              onRowsPerPageChange={handleChangeRowsPerPage}
            />
            <Dialog
              open={openTMModal}
              onClose={handleCloseTMModal}
              maxWidth="md"
              fullWidth
            >
              <DialogTitle>Order Details</DialogTitle>
              <DialogTitle>Order Id : {singleorder?.uniqueId}</DialogTitle>
              <DialogContent>
                {singleorder?.invoices?.length > 0 && (
                  <>
                    <Typography variant="h4" gutterBottom>
                      Invoices
                    </Typography>
                    <InvoiceTable invoices={singleorder.invoices} />
                  </>
                )}
                <Typography variant="h4" my={3} gutterBottom>
                  Order Summary
                </Typography>

                <Grid container spacing={2}>
                  <Grid item xs={12}>
                    <TableContainer component={Paper}>
                      <Table>
                        <TableHead>
                          <TableRow>
                            <TableCell>Product</TableCell>
                            <TableCell align="right">Price (₹)</TableCell>
                            <TableCell align="right">Quantity</TableCell>
                            <TableCell align="right">Subtotal (₹)</TableCell>
                            <TableCell align="right">GST (%)</TableCell>
                            <TableCell align="right">GST Amount (₹)</TableCell>
                            <TableCell align="right">
                              Total with GST (₹)
                            </TableCell>
                          </TableRow>
                        </TableHead>
                        <TableBody>
                          {singleorder?.orderItem.map((item, index) => {
                            const subtotal = item.price * item.quantity;
                            const gstAmount =
                              ((item.GST * item.price) / 100) * item.quantity;
                            const totalWithGST = subtotal + gstAmount;

                            return (
                              <TableRow key={index}>
                                <TableCell>
                                  <img
                                    src={item.productId.image}
                                    alt={item.productId.name}
                                    style={{
                                      width: 50,
                                      height: 50,
                                      marginRight: 10,
                                    }}
                                  />
                                  <Typography variant="subtitle1">
                                    {item.productId.name}
                                  </Typography>
                                </TableCell>
                                <TableCell align="right">
                                  ₹{item.price}
                                </TableCell>
                                <TableCell align="right">
                                  {item.quantity}
                                </TableCell>
                                <TableCell align="right">₹{subtotal}</TableCell>
                                <TableCell align="right">{item.GST}%</TableCell>
                                <TableCell align="right">
                                  ₹{gstAmount}
                                </TableCell>
                                <TableCell align="right">
                                  ₹{totalWithGST}
                                </TableCell>
                              </TableRow>
                            );
                          })}
                        </TableBody>
                      </Table>
                    </TableContainer>
                  </Grid>
                </Grid>
                {singleorder?.invoices?.length > 0 && (
                  <>
                    <Typography variant="h4" my={3} gutterBottom>
                      Order Items {singleorder?.status=="pending"?"to be Processed":"Cancelled"}
                    </Typography>
                    <PendingOrderTable order={singleorder} />
                  </>
                )}
              </DialogContent>
              <DialogActions>
                <Button onClick={handleCloseTMModal} color="secondary">
                  Cancel
                </Button>
              </DialogActions>
            </Dialog>
          </Box>
        </div>
      </div>
    </div>
  );
};
// Helper function to format time as AM/PM
const formatAMPM = (date) => {
  var hours = new Date(date).getHours();
  var minutes = new Date(date).getMinutes();
  var ampm = hours >= 12 ? "PM" : "AM";
  hours = hours % 12;
  hours = hours ? hours : 12;
  minutes = minutes < 10 ? "0" + minutes : minutes;
  var strTime = hours + ":" + minutes + " " + ampm;
  return strTime;
};
export default SinglePrincipalDistributorAllDetails;
